#
# LECA Bash library
#
# The LECA bash library provides a set of function used for helping
# development of bash script mainly to write job script on the luke
# cluster
#
# The files from the LECA Bash library must be sourced from your main script
#

include logging
logdebug "Load XML package"

include stacks


######
#
# XML formating function
#

newstack __LECABASH__markup__

function __LECABASH__XML_pad__() {
	local pad=$(printf '%0.1s' " "{1..60})
	local padlength=$(( $1 * 2 ))
	
	printf '%*.*s' 0 "$padlength" "$pad"
	
}

function XML_header() {

	echo '<?xml version="1.0" encoding="UTF-8"?>'

}

function XML_push() {

	local i=0
	local k
	local l
	local tag=$1
	shift
	local attribs=( $(echo "$*") )
	
	__LECABASH__XML_pad__ $(stacksize __LECABASH__markup__)
	echo -n "<${tag}"

	local n=${#attribs[@]}
	n=$((n/2))
	
	logdebug "XML_push ${tag} : $n attributes specified"
	
	if (( n > 0 )) ; then
		for (( i=0; i<n; i+=1 )) ; do
			k=$((i*2))
			l=$((k+1))
			echo -n " ${attribs[$k]}=\"${attribs[$l]}\""
			shift
			shift
		done
	fi
	echo ">"
	
	pushvalue __LECABASH__markup__ "${tag}"
	
}

function XML_pop() {
	local tag=""
	popvalue __LECABASH__markup__ tag

	__LECABASH__XML_pad__ $(stacksize __LECABASH__markup__)
	echo   "</${tag}>"

}

function XML_tag_data() {

	local tag=$1
	shift
	__LECABASH__XML_pad__ $(stacksize __LECABASH__markup__)
	echo   "<${tag}>$*</${tag}>"

}

function XML_tag_attr_data() {

	local i=0
	local k
	local l
	local tag=$1
	shift
	local attribs=( $(echo "$*") )

	__LECABASH__XML_pad__ $(stacksize __LECABASH__markup__)

	echo   -n "<${tag}"

	local n=${#attribs[@]}
	((n/=2))

	if (( n > 0 )) ; then
		for (( i=0; i<n; i+=1 )) ; do
			k=$((i*2))
			l=$((k+1))
			if [[ "${attribs[$k]}" != "--" ]] ; then
				echo -n " ${attribs[$k]}=\"${attribs[$l]}\""
				shift
			else
				i="$n"
			fi
			shift
		done
	fi
	
	echo ">$*</${tag}>"

}


function XML_empty_tag() {

	local tag=$1
	shift
	__LECABASH__XML_pad__ $(stacksize __LECABASH__markup__)
	echo   "<${tag}/>"

}


function XML_data() {

	__LECABASH__XML_pad__ $(stacksize __LECABASH__markup__)
	echo   "$*"

}

